/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_scanhost.c: Show users at match host and username or from server.
 *
 *  Copyright (C) 2002 by Ilya Shtift.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 */

#include "stdinc.h"
#include "tools.h"
#include "common.h"   /* bleah */
#include "handlers.h"
#include "client.h"
#include "channel.h"
#include "channel_mode.h"
#include "vchannel.h"
#include "hash.h"
#include "ircd.h"
#include "numeric.h"
#include "s_serv.h"
#include "send.h"
#include "list.h"
#include "irc_string.h"
#include "s_conf.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void mo_scanhost(struct Client*, struct Client*, int, char**);

struct Message scanhost_msgtab = {
  "SCANHOST", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_ignore, m_ignore, m_ignore, mo_scanhost}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&scanhost_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&scanhost_msgtab);
}
const char *_version = "$Revision: 1.02 $";
#endif

/*
** mo_scanhost
** Created Apr 30, 2003
** common (Ilya Shtift) ishtift@tagil.svrw.ru
**
**      parv[0] = sender
**      parv[1] = hostmask|servermask
**      parv[2] = [limit], default 100
*
*
*/
static void mo_scanhost(struct Client *client_p,
                 struct Client *source_p,
                 int parc,
                 char *parv[])
{
  struct Client *target_p;
  char *user, *host;
  char username[USERLEN + 2], hostname[HOSTLEN + 1];
  int maxmatches = 100, count = 0, broken = FALSE;

  if ( (host = strchr(parv[1], '@')) || *parv[1] == '*' )
  {
    if(host)
    {
      user = parv[1];
      *(host++) = '\0';
    }
    else
    {
      user = "*";
      host = parv[1];
    }

    if (!*host)
      host = "*";

    strlcpy(username, user, USERLEN);
    strlcpy(hostname, host, HOSTLEN);
    user = username;
    host = hostname;
  }
  else
  {
    strlcpy(hostname, parv[1], HOSTLEN);
    user = NULL;
    host = hostname;
  }

  if (parc >= 3)
  { 
    maxmatches = atoi(parv[2]);
  }

  for (target_p = GlobalClientList; target_p; target_p = target_p->next)
  {
    if (!IsPerson(target_p))
      continue;

    if ((user && (match(user, target_p->username) && match(host, target_p->host))) ||
        (!user && match(host, target_p->user->server)))
    {
      count++;
      sendto_one(source_p,":%s NOTICE %s :%s!%s@%s at %s",me.name, parv[0],
                 target_p->name, target_p->username, target_p->host, target_p->user->server);
      if (count >= maxmatches)
      {
        broken = TRUE;
        break;
      }
    }

    if (broken)
      break;
  }
  
  sendto_one(source_p,":%s NOTICE %s :%d clients match",me.name, parv[0], count);
}
