/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_alias.c: Adding aliases for services names (e.g /cs = /msg ChanServ).
 *
 *  Copyright (C) 2003 by the past and present ircd coders, and others.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_alias.c,v 1.05 2004/01/09 02:05:01 Melifaro $
 */

#include "stdinc.h"
#include "send.h"
#include "handlers.h"
#include "client.h"
#include "modules.h"
#include "numeric.h"

/* Common Services */
#define ChanServ "ChanServ"
#define NickServ "NickServ"
#define MemoServ "MemoServ"
#define OperServ "OperServ"
#define StatServ "StatServ"

/* Sentinel*/
#define MonServ "MonServ"
#define AdminServ "AdminServ"

/* Fxserv */
#define DataServ "DataServ"
		      
static void ExpandParv(char *, int, int, int, char **);
static void send_alias (struct Client*, char*, int, char**);
static void reg_id(struct Client*, char*, int, char**);
static void m_cs(struct Client*, struct Client*, int, char**);
static void m_ns(struct Client*, struct Client*, int, char**);
static void m_os(struct Client*, struct Client*, int, char**);
static void m_ss(struct Client*, struct Client*, int, char**);
static void m_ms(struct Client*, struct Client*, int, char**);
static void m_as(struct Client*, struct Client*, int, char**);
static void m_mv(struct Client*, struct Client*, int, char**);
static void m_ds(struct Client*, struct Client*, int, char**);
static void m_id(struct Client*, struct Client*, int, char**);
static void m_reg(struct Client*, struct Client*, int, char**);

struct Message cs_msgtab = {
"CS", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_cs, m_cs, m_cs, m_cs} 
};

struct Message ns_msgtab = {
"NS", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_ns, m_ns, m_ns, m_ns}
};

struct Message lcs_msgtab = {
"CHANSERV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_cs, m_cs, m_cs, m_cs}
};

struct Message lns_msgtab = {
"NICKSERV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_ns, m_ns, m_ns, m_ns}
};

struct Message ms_msgtab = {
"MS", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_ms, m_ms, m_ms, m_ms}
};

struct Message lms_msgtab = {
"MEMOSERV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_ms, m_ms, m_ms, m_ms}
};

struct Message os_msgtab = {
"OS", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_os, m_os, m_os, m_os}
};

struct Message los_msgtab = {
"OPERSERV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_os, m_os, m_os, m_os}
};

struct Message ss_msgtab = {
"SS", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_ss, m_ss, m_ss, m_ss}
};

struct Message lss_msgtab = {
"STATSERV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_ss, m_ss, m_ss, m_ss}
};

struct Message as_msgtab = {
"AS", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_as, m_as, m_as, m_as}
};

struct Message las_msgtab = {
"ADMINSERV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_as, m_as, m_as, m_as}
};

struct Message msv_msgtab = {
"MV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_mv, m_mv, m_mv, m_mv}
};

struct Message lmsv_msgtab = {
"MONSERV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_mv, m_mv, m_mv, m_mv}
};

struct Message ds_msgtab = {
"DS", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_ds, m_ds, m_ds, m_ds}
};

struct Message lds_msgtab = {
"DATASERV", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_ds, m_ds, m_ds, m_ds}
};

struct Message id_msgtab = {
"IDENTIFY", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_id, m_id, m_id, m_id}
};

struct Message reg_msgtab = {
"REGISTER", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
{m_unregistered, m_reg, m_reg, m_reg, m_reg}
};


#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&cs_msgtab);
  mod_add_cmd(&lcs_msgtab);
  mod_add_cmd(&ns_msgtab);
  mod_add_cmd(&lns_msgtab);
  mod_add_cmd(&os_msgtab);
  mod_add_cmd(&los_msgtab);
  mod_add_cmd(&ms_msgtab);
  mod_add_cmd(&lms_msgtab);
  mod_add_cmd(&ss_msgtab);
  mod_add_cmd(&lss_msgtab);
  mod_add_cmd(&as_msgtab);
  mod_add_cmd(&las_msgtab);
  mod_add_cmd(&msv_msgtab);
  mod_add_cmd(&lmsv_msgtab);
  mod_add_cmd(&ds_msgtab);
  mod_add_cmd(&lds_msgtab);
  mod_add_cmd(&id_msgtab);
  mod_add_cmd(&reg_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&cs_msgtab);
  mod_del_cmd(&lcs_msgtab);
  mod_del_cmd(&ns_msgtab);
  mod_del_cmd(&lns_msgtab);
  mod_del_cmd(&os_msgtab);
  mod_del_cmd(&los_msgtab);
  mod_del_cmd(&ms_msgtab);
  mod_del_cmd(&lms_msgtab);
  mod_del_cmd(&ss_msgtab);
  mod_del_cmd(&lss_msgtab);
  mod_del_cmd(&as_msgtab);
  mod_del_cmd(&las_msgtab);
  mod_del_cmd(&msv_msgtab);
  mod_del_cmd(&lmsv_msgtab);
  mod_del_cmd(&lds_msgtab);
  mod_del_cmd(&ds_msgtab);
  mod_del_cmd(&id_msgtab);
  mod_del_cmd(&reg_msgtab);
}

const char *_version = "$Alias mod: version 1.0.5 by Melifaro$";
#endif

static void m_cs(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
    send_alias(client_p, ChanServ, parc, parv);
}

static void m_ns(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
      send_alias(client_p, NickServ, parc, parv);
}

static void m_os(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
    send_alias(client_p, OperServ, parc, parv);
}

static void m_ss(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
    send_alias(client_p, StatServ, parc, parv);
}

static void m_ms(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
  send_alias(client_p, MemoServ, parc, parv);
}

static void m_as(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
  send_alias(client_p, AdminServ, parc, parv);
}

static void m_mv(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
  send_alias(client_p, MonServ, parc, parv);
}

static void m_ds(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
  send_alias(client_p, DataServ, parc, parv);
}

static void m_id(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
  reg_id(source_p, "IDENTIFY", parc, parv);
}

static void m_reg(struct Client *client_p, struct Client *source_p,
                  int parc, char *parv[])
{
  reg_id(source_p, "REGISTER", parc, parv);
}

static void reg_id(struct Client *source_p, char *msg, int parc, char *parv[])
{
char command[BUFSIZE];
struct Client *target_p;

if (parc < 2) return;

if (!IsPerson(source_p))
    return;

if (*parv[1] == '#') 
  {
    if (parc > 2) {
	if ((target_p = find_person(ChanServ)) != NULL)
	   {
	      snprintf(command, BUFSIZE, "%s %s %s", msg, parv[1], parv[2]);
	      sendto_anywhere(target_p, source_p,"PRIVMSG %s :%s", 
	    				target_p->name, command);	      
	   }
	  else 
	   {
	   sendto_one(source_p,
          ":%s NOTICE %s :*** Notice -- Services are currently down. "
          "Please wait a few moments, and then try again.",
          me.name, parv[0]);
	    return;
	   }
	}
  }	  		
 else
  {
    if ((target_p = find_person(NickServ)) != NULL)
      {
        snprintf(command, BUFSIZE, "%s %s", msg, parv[1]);
	sendto_anywhere(target_p, source_p,"PRIVMSG %s :%s",
				 target_p->name, command);
      }  
     else
      {
		 sendto_one(source_p,
          ":%s NOTICE %s :*** Notice -- Services are currently down. "
          "Please wait a few moments, and then try again.",
          me.name, parv[0]);
        return;
      }    
  }		      
}

static void send_alias (struct Client *source_p, char* alias, int parc, char *parv[])
{
char command[BUFSIZE];
struct Client *target_p;

if (!IsPerson(source_p))
    return;

if (parc<2) 
    {
    sendto_one(source_p, form_str(ERR_NOTEXTTOSEND), 
	    me.name, source_p->name);
    return;
    }    
    
if ((target_p = find_person(alias)) != NULL)
    {
    ExpandParv(command, BUFSIZE, 1, parc, parv);
	sendto_anywhere(target_p, source_p,"PRIVMSG %s :%s", target_p->name, command);
    return;
    }
sendto_one(source_p,
          ":%s NOTICE %s :*** Notice -- Services are currently down. "
          "Please wait a few moments, and then try again.",
          me.name, parv[0]);
}

static void ExpandParv(char *dest, int max, int initial, int parc, char *parv[])
{
  int i;
  
  if (parv[initial])
    strlcpy(dest, parv[initial], max);
  for (i = initial + 1; i < parc; i++)
  {
    strlcat(dest, " ", max);
    strlcat(dest, parv[i], max);
  }
}

